@@ -58,6 +58,6 @@ module DBF
     'c9' => 'cp1251',      # Russian Windows
     'ca' => 'cp1254',      # Turkish Windows
     'cb' => 'cp1253',      # Greek Windows
-    'cc' => 'cp1257',      # Baltic Windows
-  }
+    'cc' => 'cp1257'       # Baltic Windows
+  }.freeze
 end
