@@ -2,13 +2,19 @@ module DBF
   class FileNotFoundError < StandardError
   end
 
+  class NoColumnsDefined < StandardError
+  end
+
   # DBF::Table is the primary interface to a single DBF file and provides
   # methods for enumerating and searching the records.
   class Table
+    extend Forwardable
     include Enumerable
-    include Schema
+    include ::DBF::Schema
 
-    DBF_HEADER_SIZE = 32
+    DBASE2_HEADER_SIZE = 8
+    DBASE3_HEADER_SIZE = 32
+    DBASE7_HEADER_SIZE = 68
 
     VERSIONS = {
       '02' => 'FoxBase',
@@ -17,6 +23,7 @@ module DBF
       '05' => 'dBase V without memo file',
       '07' => 'Visual Objects 1.x',
       '30' => 'Visual FoxPro',
+      '32' => 'Visual FoxPro with field type Varchar or Varbinary',
       '31' => 'Visual FoxPro with AutoIncrement field',
       '43' => 'dBASE IV SQL table files, no memo',
       '63' => 'dBASE IV SQL system files, no memo',
@@ -24,23 +31,28 @@ module DBF
       '83' => 'dBase III with memo file',
       '87' => 'Visual Objects 1.x with memo file',
       '8b' => 'dBase IV with memo file',
+      '8c' => 'dBase 7',
       '8e' => 'dBase IV with SQL table',
       'cb' => 'dBASE IV SQL table files, with memo',
       'f5' => 'FoxPro with memo file',
       'fb' => 'FoxPro without memo file'
-    }
+    }.freeze
 
     FOXPRO_VERSIONS = {
       '30' => 'Visual FoxPro',
       '31' => 'Visual FoxPro with AutoIncrement field',
       'f5' => 'FoxPro with memo file',
       'fb' => 'FoxPro without memo file'
-    }
+    }.freeze
 
-    attr_reader :header
     attr_accessor :encoding
     attr_writer :name
 
+    def_delegator :header, :header_length
+    def_delegator :header, :record_count
+    def_delegator :header, :record_length
+    def_delegator :header, :version
+
     # Opens a DBF::Table
     # Examples:
     #   # working with a file stored on the filesystem
@@ -59,23 +71,14 @@ module DBF
     #   table = DBF::Table.new 'data.dbf', nil, 'cp437'
     #   table = DBF::Table.new 'data.dbf', 'memo.dbt', Encoding::US_ASCII
     #
-    # @param [String, StringIO] data Path to the dbf file or a StringIO object
-    # @param [optional String, StringIO] memo Path to the memo file or a StringIO object
-    # @param [optional String, Encoding] encoding Name of the encoding or an Encoding object
+    # @param data [String, StringIO] data Path to the dbf file or a StringIO object
+    # @param memo [optional String, StringIO] memo Path to the memo file or a StringIO object
+    # @param encoding [optional String, Encoding] encoding Name of the encoding or an Encoding object
     def initialize(data, memo = nil, encoding = nil)
       @data = open_data(data)
-      @data.rewind
-      @header = Header.new(@data.read DBF_HEADER_SIZE)
       @encoding = encoding || header.encoding
       @memo = open_memo(data, memo)
       yield self if block_given?
-    rescue Errno::ENOENT
-      raise DBF::FileNotFoundError, "file not found: #{data}"
-    end
-
-    # @return [TrueClass, FalseClass]
-    def has_memo_file?
-      !!@memo
     end
 
     # Closes the table and memo file
@@ -95,14 +98,18 @@ module DBF
       end
     end
 
-    # @return String
-    def filename
-      File.basename @data.path if @data.respond_to?(:path)
+    # Column names
+    #
+    # @return [String]
+    def column_names
+      @column_names ||= columns.map(&:name)
     end
 
-    # @return String
-    def name
-      @name ||= filename && File.basename(filename, ".*")
+    # All columns
+    #
+    # @return [Array]
+    def columns
+      @columns ||= build_columns
     end
 
     # Calls block once for each record in the table. The record may be nil
@@ -110,53 +117,14 @@ module DBF
     #
     # @yield [nil, DBF::Record]
     def each
-      header.record_count.times { |i| yield record(i) }
+      record_count.times { |i| yield record(i) }
     end
 
-    # Retrieve a record by index number.
-    # The record will be nil if it has been deleted, but not yet pruned from
-    # the database.
-    #
-    # @param [Fixnum] index
-    # @return [DBF::Record, NilClass]
-    def record(index)
-      seek_to_record(index)
-      return nil if deleted_record?
-      DBF::Record.new(@data.read(header.record_length), columns, version, @memo)
-    end
-
-    alias_method :row, :record
-
-    # Internal dBase version number
-    #
     # @return [String]
-    def version
-      @version ||= header.version
-    end
+    def filename
+      return unless @data.respond_to?(:path)
 
-    # Total number of records
-    #
-    # @return [Fixnum]
-    def record_count
-      @record_count ||= header.record_count
-    end
-
-    # Human readable version description
-    #
-    # @return [String]
-    def version_description
-      VERSIONS[version]
-    end
-
-    # Dumps all records to a CSV file.  If no filename is given then CSV is
-    # output to STDOUT.
-    #
-    # @param [optional String] path Defaults to STDOUT
-    def to_csv(path = nil)
-      out_io = path ? File.open(path, 'w') : $stdout
-      csv = CSV.new(out_io, force_quotes: true)
-      csv << column_names
-      each { |record| csv << record.to_a }
+      File.basename(@data.path)
     end
 
     # Find records using a simple ActiveRecord-like syntax.
@@ -181,12 +149,12 @@ module DBF
     # returned.  The equivalent SQL would be "WHERE key1 = 'value1'
     # AND key2 = 'value2'".
     #
-    # @param [Fixnum, Symbol] command
-    # @param [optional, Hash] options Hash of search parameters
+    # @param command [Integer, Symbol] command
+    # @param options [optional, Hash] options Hash of search parameters
     # @yield [optional, DBF::Record, NilClass]
     def find(command, options = {}, &block)
       case command
-      when Fixnum
+      when Integer
         record(command)
       when Array
         command.map { |i| record(i) }
@@ -197,45 +165,119 @@ module DBF
       end
     end
 
-    # All columns
-    #
-    # @return [Array]
-    def columns
-      @columns ||= build_columns
+    # @return [TrueClass, FalseClass]
+    def has_memo_file?
+      !!@memo
     end
 
-    # Column names
+    # @return [String]
+    def name
+      @name ||= filename && File.basename(filename, '.*')
+    end
+
+    # Retrieve a record by index number.
+    # The record will be nil if it has been deleted, but not yet pruned from
+    # the database.
+    #
+    # @param [Integer] index
+    # @return [DBF::Record, NilClass]
+    def record(index)
+      raise DBF::NoColumnsDefined, 'The DBF file has no columns defined' if columns.empty?
+
+      seek_to_record(index)
+      return nil if deleted_record?
+
+      record_data = @data.read(record_length)
+      DBF::Record.new(record_data, columns, version, @memo)
+    end
+
+    alias row record
+
+    # Dumps all records to a CSV file.  If no filename is given then CSV is
+    # output to STDOUT.
+    #
+    # @param [optional String] path Defaults to STDOUT
+    def to_csv(path = nil)
+      out_io = path ? File.open(path, 'w') : $stdout
+      csv = CSV.new(out_io, force_quotes: true)
+      csv << column_names
+      each { |record| csv << record.to_a }
+    end
+
+    # Human readable version description
     #
     # @return [String]
-    def column_names
-      columns.map(&:name)
+    def version_description
+      VERSIONS[version]
     end
 
     private
 
-    def build_columns # nodoc
-      @data.seek(DBF_HEADER_SIZE)
-      columns = []
-      until end_of_record?
-        column_data = @data.read(DBF_HEADER_SIZE)
-        name, type, length, decimal = column_data.unpack('a10 x a x4 C2')
-        columns << Column.new(self, name, type, length, decimal)
+    def build_columns # :nodoc:
+      safe_seek do
+        @data.seek(header_size)
+        [].tap do |columns|
+          until end_of_record?
+            args = case version
+            when '02'
+              [self, *@data.read(header_size * 2).unpack('A11 a C'), 0]
+            when '04', '8c'
+              [self, *@data.read(48).unpack('A32 a C C x13')]
+            else
+              [self, *@data.read(header_size).unpack('A11 a x4 C2')]
+            end
+
+            columns << Column.new(*args)
+          end
+        end
       end
-      columns
     end
 
-    def end_of_record? # nodoc
-      original_pos = @data.pos
-      byte = @data.read(1)
-      @data.seek(original_pos)
-      byte.ord == 13
+    def header_size
+      case version
+      when '02'
+        DBASE2_HEADER_SIZE
+      when '04', '8c'
+        DBASE7_HEADER_SIZE
+      else 
+        DBASE3_HEADER_SIZE
+      end
     end
 
-    def foxpro? # nodoc
-      FOXPRO_VERSIONS.keys.include? version
+    def deleted_record? # :nodoc:
+      flag = @data.read(1)
+      flag ? flag.unpack1('a') == '*' : true
     end
 
-    def memo_class # nodoc
+    def end_of_record? # :nodoc:
+      safe_seek { @data.read(1).ord == 13 }
+    end
+
+    def find_all(options) # :nodoc:
+      select do |record|
+        next unless record && record.match?(options)
+
+        yield record if block_given?
+        record
+      end
+    end
+
+    def find_first(options) # :nodoc:
+      detect { |record| record && record.match?(options) }
+    end
+
+    def foxpro? # :nodoc:
+      FOXPRO_VERSIONS.key?(version)
+    end
+
+    def header # :nodoc:
+      @header ||= safe_seek do
+        @data.seek(0)
+        Header.new(@data.read(DBASE3_HEADER_SIZE))
+      end
+    end
+
+    def memo_class # :nodoc:
       @memo_class ||= begin
         if foxpro?
           Memo::Foxpro
@@ -245,49 +287,39 @@ module DBF
       end
     end
 
-    def open_data(data) # nodoc
-      data.is_a?(StringIO) ? data : File.open(data, 'rb')
-    end
-
-    def open_memo(data, memo = nil) # nodoc
-      if memo
-        meth = memo.is_a?(StringIO) ? :new : :open
-        memo_class.send(meth, memo, version)
-      elsif !data.is_a? StringIO
-        files = Dir.glob(memo_search_path data)
-        files.any? ? memo_class.open(files.first, version) : nil
-      end
-    end
-
-    def memo_search_path(io) # nodoc
+    def memo_search_path(io) # :nodoc:
       dirname = File.dirname(io)
       basename = File.basename(io, '.*')
       "#{dirname}/#{basename}*.{fpt,FPT,dbt,DBT}"
     end
 
-    def find_all(options) # nodoc
-      map do |record|
-        if record && record.match?(options)
-          yield record if block_given?
-          record
-        end
-      end.compact
+    def open_data(data) # :nodoc:
+      data.is_a?(StringIO) ? data : File.open(data, 'rb')
+    rescue Errno::ENOENT
+      raise DBF::FileNotFoundError, "file not found: #{data}"
     end
 
-    def find_first(options) # nodoc
-      detect { |record| record && record.match?(options) }
+    def open_memo(data, memo = nil) # :nodoc:
+      if memo
+        meth = memo.is_a?(StringIO) ? :new : :open
+        memo_class.send(meth, memo, version)
+      elsif !data.is_a?(StringIO)
+        files = Dir.glob(memo_search_path(data))
+        files.any? ? memo_class.open(files.first, version) : nil
+      end
     end
 
-    def deleted_record? # nodoc
-      @data.read(1).unpack('a') == ['*']
+    def safe_seek # :nodoc:
+      original_pos = @data.pos
+      yield.tap { @data.seek(original_pos) }
     end
 
-    def seek(offset) # nodoc
-      @data.seek header.header_length + offset
+    def seek(offset) # :nodoc:
+      @data.seek(header_length + offset)
     end
 
-    def seek_to_record(index) # nodoc
-      seek(index * header.record_length)
+    def seek_to_record(index) # :nodoc:
+      seek(index * record_length)
     end
   end
 end
