@@ -1,4 +1,4 @@
-require "spec_helper"
+require 'spec_helper'
 
 RSpec.describe DBF::Record do
   describe '#to_a' do
@@ -6,7 +6,7 @@ RSpec.describe DBF::Record do
     let(:record_0) { YAML.load_file(fixture('dbase_83_record_0.yml')) }
     let(:record_9) { YAML.load_file(fixture('dbase_83_record_9.yml')) }
 
-    it 'should return an ordered array of attribute values' do
+    it 'returns an ordered array of attribute values' do
       record = table.record(0)
       expect(record.to_a).to eq record_0
 
@@ -44,13 +44,13 @@ RSpec.describe DBF::Record do
 
     describe 'when other does not have attributes' do
       it 'returns false' do
-        expect((record == double('other'))).to be_falsey
+        expect((record == instance_double('DBF::Record'))).to be_falsey
       end
     end
 
     describe 'if other attributes match' do
       let(:attributes) { {x: 1, y: 2} }
-      let(:other) { double('object', attributes: attributes) }
+      let(:other) { instance_double('DBF::Record', attributes: attributes) }
 
       before do
         allow(record).to receive(:attributes).and_return(attributes)
@@ -67,8 +67,7 @@ RSpec.describe DBF::Record do
     let(:table) { DBF::Table.new fixture('dbase_8b.dbf') }
     let(:record) { table.find(0) }
 
-    %w(character numerical date logical float memo).each do |column_name|
-
+    %w[character numerical date logical float memo].each do |column_name|
       it "defines accessor method for '#{column_name}' column" do
         expect(record).to respond_to(column_name.to_sym)
       end
@@ -81,23 +80,23 @@ RSpec.describe DBF::Record do
       let(:table) { DBF::Table.new fixture('cp1251.dbf') }
       let(:record) { table.find(0) }
 
-      it 'should automatically encodes to default system encoding' do
+      it 'encodes to default system encoding' do
         expect(record.name.encoding).to eq Encoding.default_external
 
         # russian a
-        expect(record.name.encode("UTF-8").unpack("H4")).to eq ["d0b0"]
+        expect(record.name.encode('UTF-8').unpack1('H4')).to eq 'd0b0'
       end
     end
 
     describe 'overriding specified in dbf encoding' do
-      let(:table) { DBF::Table.new fixture('cp1251.dbf'), nil, 'cp866'}
+      let(:table) { DBF::Table.new fixture('cp1251.dbf'), nil, 'cp866' }
       let(:record) { table.find(0) }
 
-      it 'should transcode from manually specified encoding to default system encoding' do
+      it 'transcodes from manually specified encoding to default system encoding' do
         expect(record.name.encoding).to eq Encoding.default_external
 
         # russian а encoded in cp1251 and read as if it was encoded in cp866
-        expect(record.name.encode("UTF-8").unpack("H4")).to eq ["d180"]
+        expect(record.name.encode('UTF-8').unpack1('H4')).to eq 'd180'
       end
     end
   end
@@ -112,7 +111,7 @@ RSpec.describe DBF::Record do
     end
 
     it 'has only original field names as keys' do
-      original_field_names = %w(CHARACTER DATE FLOAT LOGICAL MEMO NUMERICAL)
+      original_field_names = %w[CHARACTER DATE FLOAT LOGICAL MEMO NUMERICAL]
       expect(record.attributes.keys.sort).to eq original_field_names
     end
   end
