@@ -6,7 +6,11 @@ RSpec.describe DBF::Table do
   let(:table) { DBF::Table.new dbf_path }
 
   specify 'foxpro versions' do
-    expect(DBF::Table::FOXPRO_VERSIONS.keys.sort).to eq %w(30 31 f5 fb).sort
+    expect(DBF::Table::FOXPRO_VERSIONS.keys.sort).to eq %w[30 31 f5 fb].sort
+  end
+
+  specify 'row is an alias of record' do
+    expect(table.record(1)).to eq table.row(1)
   end
 
   describe '#initialize' do
@@ -40,7 +44,7 @@ RSpec.describe DBF::Table do
     end
   end
 
-  context '#close' do
+  describe '#close' do
     before { table.close }
 
     it 'closes the io' do
@@ -50,11 +54,18 @@ RSpec.describe DBF::Table do
 
   describe '#schema' do
     describe 'when data is IO' do
-      let(:control_schema) { File.read(fixture('dbase_83_schema.txt')) }
+      let(:control_schema) { File.read(fixture('dbase_83_schema_ar.txt')) }
 
       it 'matches the test schema fixture' do
         expect(table.schema).to eq control_schema
       end
+
+      it 'raises ArgumentError if there is no matching schema' do
+        expect { table.schema(:invalid) }.to raise_error(
+          ArgumentError,
+          ':invalid is not a valid schema. Valid schemas are: activerecord, json, sequel.'
+        )
+      end
     end
 
     describe 'when data is StringIO' do
@@ -62,7 +73,7 @@ RSpec.describe DBF::Table do
       let(:memo) { StringIO.new File.read(memo_path) }
       let(:table) { DBF::Table.new data }
 
-      let(:control_schema) { File.read(fixture('dbase_83_schema.txt')) }
+      let(:control_schema) { File.read(fixture('dbase_83_schema_ar.txt')) }
 
       it 'matches the test schema fixture' do
         table.name = 'dbase_83'
@@ -72,52 +83,14 @@ RSpec.describe DBF::Table do
   end
 
   describe '#sequel_schema' do
-    it 'should return a valid Sequel migration by default' do
-      expect(table.sequel_schema).to eq <<-SCHEMA
-Sequel.migration do
-  change do
-     create_table(:dbase_83) do
-      column :id, :integer
-      column :catcount, :integer
-      column :agrpcount, :integer
-      column :pgrpcount, :integer
-      column :order, :integer
-      column :code, :varchar, :size => 50
-      column :name, :varchar, :size => 100
-      column :thumbnail, :varchar, :size => 254
-      column :image, :varchar, :size => 254
-      column :price, :float
-      column :cost, :float
-      column :desc, :text
-      column :weight, :float
-      column :taxable, :boolean
-      column :active, :boolean
-    end
-  end
-end
-SCHEMA
+    it 'returns a valid Sequel migration by default' do
+      control_schema = File.read(fixture('dbase_83_schema_sq.txt'))
+      expect(table.sequel_schema).to eq control_schema
     end
 
-    it 'should return a limited Sequel migration when passed true' do
-      expect(table.sequel_schema(true)).to eq <<-SCHEMA
-    create_table(:dbase_83) do
-      column :id, :integer
-      column :catcount, :integer
-      column :agrpcount, :integer
-      column :pgrpcount, :integer
-      column :order, :integer
-      column :code, :varchar, :size => 50
-      column :name, :varchar, :size => 100
-      column :thumbnail, :varchar, :size => 254
-      column :image, :varchar, :size => 254
-      column :price, :float
-      column :cost, :float
-      column :desc, :text
-      column :weight, :float
-      column :taxable, :boolean
-      column :active, :boolean
-    end
-SCHEMA
+    it 'returns a limited Sequel migration when passed true' do
+      control_schema = File.read(fixture('dbase_83_schema_sq_lim.txt'))
+      expect(table.sequel_schema).to eq control_schema
     end
 
   end
@@ -156,13 +129,7 @@ SCHEMA
 
     describe 'when no path param passed' do
       it 'writes to STDOUT' do
-        begin
-          $stdout = StringIO.new
-          table.to_csv
-          expect($stdout.string).not_to be_empty
-        ensure
-          $stdout = STDOUT
-        end
+        expect { table.to_csv }.to output.to_stdout
       end
     end
 
@@ -170,7 +137,7 @@ SCHEMA
       before { table.to_csv('test.csv') }
 
       it 'creates a custom csv file' do
-        expect(File.exist?('test.csv')).to be_truthy
+        expect(File).to be_exist('test.csv')
       end
     end
   end
@@ -180,10 +147,18 @@ SCHEMA
       allow(table).to receive(:deleted_record?).and_return(true)
       expect(table.record(5)).to be_nil
     end
+
+    describe 'when dbf has no column definitions' do
+      let(:dbf_path) { fixture('polygon.dbf') }
+
+      it 'raises a DBF::NoColumnsDefined error' do
+        expect { DBF::Table.new(dbf_path).record(1) }.to raise_error(DBF::NoColumnsDefined, 'The DBF file has no columns defined')
+      end
+    end
   end
 
   describe '#current_record' do
-    it 'should return nil for deleted records' do
+    it 'returns nil for deleted records' do
       allow(table).to receive(:deleted_record?).and_return(true)
       expect(table.record(0)).to be_nil
     end
@@ -204,13 +179,20 @@ SCHEMA
 
     describe 'with :all' do
       let(:records) do
-        table.find(:all, weight: 0.0).inject([]) do |records, record|
-          records << record
-        end
+        table.find(:all, weight: 0.0)
       end
 
-      it 'accepts a block' do
-        expect(records).to eq table.find(:all, weight: 0.0)
+      it 'retrieves only matching records' do
+        expect(records.size).to eq 66
+      end
+
+      it 'yields to a block if given' do
+        record_count = 0
+        table.find(:all, weight: 0.0) do |record|
+          record_count += 1
+          expect(record).to be_a DBF::Record
+        end
+        expect(record_count).to eq 66
       end
 
       it 'returns all records if options are empty' do
@@ -218,27 +200,27 @@ SCHEMA
       end
 
       it 'returns matching records when used with options' do
-        expect(table.find(:all, 'WEIGHT' => 0.0)).to eq table.select {|r| r['weight'] == 0.0}
+        expect(table.find(:all, 'WEIGHT' => 0.0)).to eq table.select { |r| r['weight'] == 0.0 }
       end
 
-      it 'should AND multiple search terms' do
+      it 'ANDS multiple search terms' do
         expect(table.find(:all, 'ID' => 30, :IMAGE => 'graphics/00000001/TBC01.jpg')).to be_empty
       end
 
-      it 'should match original column names' do
-        expect(table.find(:all, 'WEIGHT' => 0.0)).not_to be_empty
+      it 'matches original column names' do
+        expect(table.find(:all, 'WEIGHT' => 0.0)).to_not be_empty
       end
 
       it 'matches symbolized column names' do
-        expect(table.find(:all, :WEIGHT => 0.0)).not_to be_empty
+        expect(table.find(:all, WEIGHT: 0.0)).to_not be_empty
       end
 
       it 'matches downcased column names' do
-        expect(table.find(:all, 'weight' => 0.0)).not_to be_empty
+        expect(table.find(:all, 'weight' => 0.0)).to_not be_empty
       end
 
       it 'matches symbolized downcased column names' do
-        expect(table.find(:all, :weight => 0.0)).not_to be_empty
+        expect(table.find(:all, weight: 0.0)).to_not be_empty
       end
     end
 
@@ -296,13 +278,13 @@ SCHEMA
       let(:table) { DBF::Table.new fixture('dbase_03.dbf') }
 
       it 'is false' do
-        expect(table.has_memo_file?).to be_falsey
+        expect(table).to_not have_memo_file
       end
     end
 
     describe 'with a memo file' do
       it 'is true' do
-        expect(table.has_memo_file?).to be_truthy
+        expect(table).to have_memo_file
       end
     end
   end
@@ -313,17 +295,76 @@ SCHEMA
     it 'is an array of Columns' do
       expect(columns).to be_an(Array)
       expect(columns).to_not be_empty
-      expect(columns.all? {|c| c.class == DBF::Column}).to be_truthy
+      expect(columns).to be_all { |c| c.is_a? DBF::Column }
     end
   end
 
   describe '#column_names' do
     let(:column_names) do
-      %w(ID CATCOUNT AGRPCOUNT PGRPCOUNT ORDER CODE NAME THUMBNAIL IMAGE PRICE COST DESC WEIGHT TAXABLE ACTIVE)
+      %w[ID CATCOUNT AGRPCOUNT PGRPCOUNT ORDER CODE NAME THUMBNAIL IMAGE PRICE COST DESC WEIGHT TAXABLE ACTIVE]
     end
 
-    it 'is an array of all column names' do
-      expect(table.column_names).to eq column_names
+    describe 'when data is an IO' do
+      it 'is an array of all column names' do
+        expect(table.column_names).to eq column_names
+      end
+    end
+
+    describe 'when data is a StringIO' do
+      let(:data) { StringIO.new File.read(dbf_path) }
+      let(:table) { DBF::Table.new data, nil, Encoding::US_ASCII }
+
+      it 'is an array of all column names' do
+        expect(table.column_names).to eq column_names
+      end
+    end
+  end
+
+  describe '#activerecord_schema_definition' do
+    context 'with type N (number)' do
+      it 'outputs an integer column' do
+        column = DBF::Column.new table, 'ColumnName', 'N', 1, 0
+        expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :integer\n"
+      end
+    end
+
+    describe 'with type B (binary)' do
+      context 'with Foxpro dbf' do
+        it 'outputs a float column' do
+          column = DBF::Column.new table, 'ColumnName', 'B', 1, 2
+          expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :binary\n"
+        end
+      end
+    end
+
+    it 'defines a float colmn if type is (N)umber with more than 0 decimals' do
+      column = DBF::Column.new table, 'ColumnName', 'N', 1, 2
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :float\n"
+    end
+
+    it 'defines a date column if type is (D)ate' do
+      column = DBF::Column.new table, 'ColumnName', 'D', 8, 0
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :date\n"
+    end
+
+    it 'defines a datetime column if type is (D)ate' do
+      column = DBF::Column.new table, 'ColumnName', 'T', 16, 0
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :datetime\n"
+    end
+
+    it 'defines a boolean column if type is (L)ogical' do
+      column = DBF::Column.new table, 'ColumnName', 'L', 1, 0
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :boolean\n"
+    end
+
+    it 'defines a text column if type is (M)emo' do
+      column = DBF::Column.new table, 'ColumnName', 'M', 1, 0
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :text\n"
+    end
+
+    it 'defines a string column with length for any other data types' do
+      column = DBF::Column.new table, 'ColumnName', 'X', 20, 0
+      expect(table.activerecord_schema_definition(column)).to eq "\"column_name\", :string, :limit => 20\n"
     end
   end
 end
