@@ -1,6 +1,6 @@
 begin
-  require 'codeclimate-test-reporter'
-  CodeClimate::TestReporter.start
+  require 'simplecov'
+  SimpleCov.start
 rescue LoadError
 end
 
@@ -24,7 +24,7 @@ RSpec.configure do |config|
 end
 
 def fixture_path
-  @fixture_path ||= File.join(File.dirname(__FILE__), '/fixtures')
+  @fixture_path ||= File.join(File.dirname(__FILE__), 'fixtures')
 end
 
 def fixture(filename)
